@@ -20,16 +20,17 @@
 import weakref
 import logging
 
+import dbus
 import telepathy
-import pymsn
-import pymsn.event
+import papyon
+import papyon.event
 
 from butterfly.presence import ButterflyPresence
-from butterfly.simple_presence import ButterflySimplePresence
 from butterfly.aliasing import ButterflyAliasing
 from butterfly.avatars import ButterflyAvatars
 from butterfly.handle import ButterflyHandleFactory
-from butterfly.channel_manager import ChannelManager
+from butterfly.contacts import ButterflyContacts
+from butterfly.channel_manager import ButterflyChannelManager
 
 __all__ = ['ButterflyConnection']
 
@@ -37,12 +38,13 @@ logger = logging.getLogger('Butterfly.Connection')
 
 
 class ButterflyConnection(telepathy.server.Connection,
+        telepathy.server.ConnectionInterfaceRequests,
         ButterflyPresence,
-        ButterflySimplePresence,
         ButterflyAliasing,
         ButterflyAvatars,
-        pymsn.event.ClientEventInterface,
-        pymsn.event.InviteEventInterface):
+        ButterflyContacts,
+        papyon.event.ClientEventInterface,
+        papyon.event.InviteEventInterface):
 
     _mandatory_parameters = {
             'account' : 's',
@@ -82,27 +84,25 @@ class ButterflyConnection(telepathy.server.Connection,
                 proxies['https'] = proxy
 
             self._manager = weakref.proxy(manager)
-            self._msn_client = pymsn.Client(server, proxies)
+            self._msn_client = papyon.Client(server, proxies)
             self._account = (parameters['account'].encode('utf-8'),
                     parameters['password'].encode('utf-8'))
-            self._channel_manager = ChannelManager(self)
+            self._channel_manager = ButterflyChannelManager(self)
 
             # Call parent initializers
-            try:
-                telepathy.server.Connection.__init__(self, 'msn', account, 'butterfly')
-            except TypeError: # handle old versions of tp-python
-                telepathy.server.Connection.__init__(self, 'msn', account)
+            telepathy.server.Connection.__init__(self, 'msn', account, 'butterfly')
+            telepathy.server.ConnectionInterfaceRequests.__init__(self)
             ButterflyPresence.__init__(self)
-            ButterflySimplePresence.__init__(self)
             ButterflyAliasing.__init__(self)
             ButterflyAvatars.__init__(self)
-            pymsn.event.ClientEventInterface.__init__(self, self._msn_client)
-            pymsn.event.InviteEventInterface.__init__(self, self._msn_client)
+            ButterflyContacts.__init__(self)
+            papyon.event.ClientEventInterface.__init__(self, self._msn_client)
+            papyon.event.InviteEventInterface.__init__(self, self._msn_client)
 
             self.set_self_handle(ButterflyHandleFactory(self, 'self'))
 
             self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NONE_SPECIFIED
-            self._initial_presence = None
+            self._initial_presence = papyon.Presence.INVISIBLE
             self._initial_personal_message = None
 
             logger.info("Connection to the account %s created" % account)
@@ -144,9 +144,9 @@ class ButterflyConnection(telepathy.server.Connection,
                 name = name.rsplit('#', 1)
                 contact_name = name[0]
                 if len(name) > 1:
-                    network_id = int(contact_name[1])
+                    network_id = int(name[1])
                 else:
-                    network_id = pymsn.NetworkID.MSN
+                    network_id = papyon.NetworkID.MSN
                 contacts = self.msn_client.address_book.contacts.\
                         search_by_account(contact_name).\
                         search_by_network_id(network_id)
@@ -168,48 +168,70 @@ class ButterflyConnection(telepathy.server.Connection,
             self.add_client_handle(handle, sender)
         return handles
 
-    def RequestChannel(self, type, handle_type, handle_id, suppress_handler):
+    def _generate_props(self, channel_type, handle, suppress_handler):
+        return {
+            telepathy.CHANNEL_INTERFACE + '.ChannelType': channel_type,
+            telepathy.CHANNEL_INTERFACE + '.TargetHandle': 0 if handle is None else handle.get_id(),
+            telepathy.CHANNEL_INTERFACE + '.TargetHandleType': telepathy.HANDLE_TYPE_NONE if handle is None else handle.get_type(),
+            telepathy.CHANNEL_INTERFACE + '.Requested': suppress_handler
+            }
+
+    @dbus.service.method(telepathy.CONNECTION, in_signature='suub',
+        out_signature='o', async_callbacks=('_success', '_error'))
+    def RequestChannel(self, type, handle_type, handle_id, suppress_handler,
+            _success, _error):
         self.check_connected()
-
-        channel = None
         channel_manager = self._channel_manager
-        handle = self.handle(handle_type, handle_id)
 
-        if type == telepathy.CHANNEL_TYPE_CONTACT_LIST:
-            channel = channel_manager.channel_for_list(handle, suppress_handler)
-        elif type == telepathy.CHANNEL_TYPE_TEXT:
-            if handle_type != telepathy.HANDLE_TYPE_CONTACT:
-                raise telepathy.NotImplemented("Only Contacts are allowed")
-            contact = handle.contact
-            if contact.presence == pymsn.Presence.OFFLINE:
-                raise telepathy.NotAvailable('Contact not available')
-            channel = channel_manager.channel_for_text(handle, None, suppress_handler)
+        if handle_id == 0:
+            handle = None
         else:
-            raise telepathy.NotImplemented("unknown channel type %s" % type)
+            handle = self.handle(handle_type, handle_id)
+        props = self._generate_props(type, handle, suppress_handler)
+        self._validate_handle(props)
 
-        return channel._object_path
+        channel = channel_manager.channel_for_props(props, signal=False)
 
-    # pymsn.event.ClientEventInterface
+        _success(channel._object_path)
+        self.signal_new_channels([channel])
+
+    # papyon.event.ClientEventInterface
     def on_client_state_changed(self, state):
-        if state == pymsn.event.ClientState.CONNECTING:
+        if state == papyon.event.ClientState.CONNECTING:
             self.StatusChanged(telepathy.CONNECTION_STATUS_CONNECTING,
                     telepathy.CONNECTION_STATUS_REASON_REQUESTED)
-        elif state == pymsn.event.ClientState.SYNCHRONIZED:
+        elif state == papyon.event.ClientState.SYNCHRONIZED:
             handle = ButterflyHandleFactory(self, 'list', 'subscribe')
-            self._channel_manager.channel_for_list(handle)
+            props = self._generate_props(telepathy.CHANNEL_TYPE_CONTACT_LIST,
+                handle, False)
+            self._channel_manager.channel_for_props(props, signal=True)
+
             handle = ButterflyHandleFactory(self, 'list', 'publish')
-            self._channel_manager.channel_for_list(handle)
+            props = self._generate_props(telepathy.CHANNEL_TYPE_CONTACT_LIST,
+                handle, False)
+            self._channel_manager.channel_for_props(props, signal=True)
+
             #handle = ButterflyHandleFactory(self, 'list', 'hide')
-            #self._channel_manager.channel_for_list(handle)
+            #props = self._generate_props(telepathy.CHANNEL_TYPE_CONTACT_LIST,
+            #    handle, False)
+            #self._channel_manager.channel_for_props(props, signal=True)
+
             #handle = ButterflyHandleFactory(self, 'list', 'allow')
-            #self._channel_manager.channel_for_list(handle)
+            #props = self._generate_propstelepathy.CHANNEL_TYPE_CONTACT_LIST,
+            #    handle, False)
+            #self._channel_manager.channel_for_props(props, signal=True)
+
             #handle = ButterflyHandleFactory(self, 'list', 'deny')
-            #self._channel_manager.channel_for_list(handle)
+            #props = self._generate_props(telepathy.CHANNEL_TYPE_CONTACT_LIST,
+            #    handle, False)
+            #self._channel_manager.channel_for_props(props, signal=True)
 
             for group in self.msn_client.address_book.groups:
                 handle = ButterflyHandleFactory(self, 'group', group.name)
-                self._channel_manager.channel_for_list(handle)
-        elif state == pymsn.event.ClientState.OPEN:
+                props = self._generate_props(
+                    telepathy.CHANNEL_TYPE_CONTACT_LIST, handle, False)
+                self._channel_manager.channel_for_props(props, signal=True)
+        elif state == papyon.event.ClientState.OPEN:
             self.StatusChanged(telepathy.CONNECTION_STATUS_CONNECTED,
                     telepathy.CONNECTION_STATUS_REASON_REQUESTED)
             presence = self._initial_presence
@@ -223,33 +245,40 @@ class ButterflyConnection(telepathy.server.Connection,
                 self._presence_changed(ButterflyHandleFactory(self, 'self'),
                         self._client.profile.presence,
                         self._client.profile.personal_message)
-        elif state == pymsn.event.ClientState.CLOSED:
+        elif state == papyon.event.ClientState.CLOSED:
             self.StatusChanged(telepathy.CONNECTION_STATUS_DISCONNECTED,
                     self.__disconnect_reason)
             #FIXME
             self._channel_manager.close()
             self._advertise_disconnected()
 
-    # pymsn.event.ClientEventInterface
+    # papyon.event.ClientEventInterface
     def on_client_error(self, type, error):
-        if type == pymsn.event.ClientErrorType.NETWORK:
+        if type == papyon.event.ClientErrorType.NETWORK:
             self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NETWORK_ERROR
-        elif type == pymsn.event.ClientErrorType.AUTHENTICATION:
+        elif type == papyon.event.ClientErrorType.AUTHENTICATION:
             self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED
+        elif type == papyon.event.ClientErrorType.PROTOCOL and \
+             error == papyon.event.ProtocolError.OTHER_CLIENT:
+            self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NAME_IN_USE
         else:
             self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_NONE_SPECIFIED
 
-    # pymsn.event.InviteEventInterface
+    # papyon.event.InviteEventInterface
     def on_invite_conversation(self, conversation):
         logger.debug("Conversation invite")
         #FIXME: get rid of this crap and implement group support
-        participants = conversation.participants 
+        participants = conversation.participants
         for p in participants:
             participant = p
             break
         handle = ButterflyHandleFactory(self, 'contact',
                 participant.account, participant.network_id)
-        channel = self._channel_manager.channel_for_text(handle, conversation)
+
+        props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT,
+            handle, False)
+        channel = self._channel_manager.channel_for_props(props,
+            signal=True, conversation=conversation)
 
     def _advertise_disconnected(self):
         self._manager.disconnected(self)
@@ -261,7 +290,7 @@ def build_proxy_infos(self, parameters, proxy_type='http'):
     username_key = proxy_type + '-proxy-username'
     password_key = proxy_type + '-proxy-password'
     if server_key in parameters and port_key in parameters:
-        return pymsn.ProxyInfos(host = parameters[server_key],
+        return papyon.ProxyInfos(host = parameters[server_key],
                 port = parameters[port_key],
                 type = proxy_type,
                 user = parameters.get(username_key, None),
